import {
  Application,
  Container,
  EventEmitter,
  Graphics,
  Sprite,
  Text,
  Texture,
  Ticker
} from 'pixi.js'
import { Tile } from './Tile'
import type { PlayerState, TileDto } from './interfaces'
import { GlowFilter } from 'pixi-filters'

export default class Hand extends EventEmitter {
  tiles: Tile[] = []
  container: Container = new Container()
  buttonPassContainer: Container = new Container()
  height: number
  width: number
  ticker: Ticker
  lastTimeClicked: number = 0
  doubleClickThreshold: number = 300
  initialized: boolean = false
  _canMove: boolean = false
  scale: number = 1

  constructor(app: Application) {
    super()
    app.stage.addChild(this.container)
    this.ticker = app.ticker
    this.height = 130
    this.width = app.canvas.width
    this.container.y = app.canvas.height - this.height
    this.container.width = this.width
    this.container.height = this.height
    this.addBg()
    this.createPassButton()
  }

  get canMove() {
    return this._canMove
  }

  set canMove(value: boolean) {
    this._canMove = value
    this.buttonPassContainer.eventMode = value ? 'static' : 'none'
    this.buttonPassContainer.cursor = value ? 'pointer' : 'default'
    this.tiles.forEach((tile) => {
      tile.interactive = value
    })
  }

  initialize(playerState: PlayerState) {
    this.tiles = this.createTiles(playerState)
    this.emit('handUpdated', this.tiles)
    this.initialized = this.tiles.length > 0
    this.renderTiles()
  }

  private addBg() {
    const bg = new Sprite(Texture.WHITE)
    bg.alpha = 0.08
    bg.width = this.width
    bg.height = this.height
    this.container.addChild(bg)
  }

  private onTileClick(tile: Tile) {
    // if (Date.now() - this.lastTimeClicked < this.doubleClickThreshold) {
    //   this.emit('tileDoubleClick', { id: tile.id })
    //   return
    // }

    const selected = this.tiles.find((t) => t.selected)
    if (selected) {
      this.deselectTile(selected)
      if (selected.id === tile.id) {
        this.emit('tileClick')
        return
      }
    }

    tile.selected = true
    tile.alpha = 1

    tile.animateTo(tile.x, tile.y - 10)
    this.emit('tileClick', tile.toPlain())
  }

  private deselectTile(selected: Tile) {
    selected.animateTo(selected.x, selected.y + 10)
    selected.selected = false
    selected.alpha = 0.7
  }

  public tileMoved(tileDto: TileDto) {
    const tile = this.tiles.find((t) => t.id === tileDto.id)

    if (!tile) return

    tile.interactive = false
    tile.clearFilters()
    tile.off('pointerdown')
    tile.off('pointerover')
    tile.off('pointerout')
  }

  private createPassButton() {
    const rectangle = new Graphics().roundRect(0, 0, 80, 30, 10).fill(0xffff00)

    const text = new Text({
      text: 'PASS',
      style: {
        fontFamily: 'Arial',
        fontSize: 12,
        fontWeight: 'bold',
        fill: 0x121212,
        align: 'center'
      }
    })
    text.anchor = 0.5

    this.buttonPassContainer = new Container()

    this.buttonPassContainer.addChild(rectangle)
    this.buttonPassContainer.addChild(text)

    text.y = this.buttonPassContainer.height / 2 - 4
    text.x = this.buttonPassContainer.width / 2 - 8

    this.buttonPassContainer.eventMode = 'none'
    this.buttonPassContainer.cursor = 'default'
    this.buttonPassContainer.x = 20
    this.buttonPassContainer.y = this.height / 2 - 10
    rectangle.alpha = 0.7
    text.alpha = 0.7
    this.buttonPassContainer.on('pointerdown', () => {
      this.emit('passClick')
    })
    this.buttonPassContainer.on('pointerover', () => {
      rectangle.alpha = 1
      text.alpha = 1
    })

    this.buttonPassContainer.on('pointerout', () => {
      rectangle.alpha = 0.7
      text.alpha = 0.7
    })

    this.container.addChild(this.buttonPassContainer)
  }

  update(playerState: PlayerState) {
    if (!this.initialized) {
      this.initialize(playerState)
      return
    }
    const missing: Tile | undefined = this.tiles.find(
      (tile: Tile) => !playerState.hand.find((t) => t.id === tile.id)
    )
    if (missing) {
      this.container.removeChild(missing.getSprite())
      this.tiles = this.tiles.filter((tile) => tile.id !== missing.id)
      this.emit('handUpdated', this.tiles)
    }
    this.renderTiles()
  }

  private createTiles(playerState: PlayerState) {
    return playerState.hand.map((tile, i) => {
      const newTile: Tile = new Tile(tile.id, this.ticker, tile.pips, this.scale)
      newTile.alpha = 0.7
      newTile.anchor = 0.5
      newTile.addTo(this.container)
      newTile.on('pointerdown', () => this.onTileClick(newTile))
      newTile.on('pointerover', () => {
        newTile.alpha = 1
        newTile.setFilters([
          new GlowFilter({
            distance: 10,
            outerStrength: 2,
            innerStrength: 1,
            color: 0xffffff,
            quality: 0.5
          })
        ])
      })
      newTile.on('pointerout', () => {
        if (!newTile.selected) {
          newTile.alpha = 0.7
          newTile.getSprite().filters = []
        }
      })
      return newTile
    })
  }

  renderTiles() {
    const deltaX = this.width / 2 - (this.tiles.length * 50 - 5) / 2
    this.tiles.forEach((tile, i) => {
      tile.setPosition(deltaX + tile.width / 2 + i * (tile.width + 5), tile.height / 2 + 20)
    })
  }
}
